==> gumga-application-2.0.1-sources.jar: io/gumga/application/nlp/GumgaNLP.java <==
package io.gumga.application.nlp;

import io.gumga.core.GumgaValues;
import io.gumga.domain.nlp.GumgaNLPThing;
import org.cogroo.analyzer.Analyzer;
import org.cogroo.analyzer.ComponentFactory;
import org.cogroo.text.*;
import org.cogroo.text.impl.DocumentImpl;
import org.reflections.Reflections;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Component
public class GumgaNLP {
    
    public static final String NO_GUMGANLP = "NO_GUMGANLP";

    public static final String NO_NAME = "_NO_NAME";
    private Analyzer cogroo;
    private Reflections reflections;
    private Set<Class<?>> classOfInterest;
    private static final Logger log = LoggerFactory.getLogger(GumgaNLP.class);

    @Autowired
    public GumgaNLP(GumgaValues gumgaValues) {
        String basePackage = gumgaValues.getGumgaNLPBasePackage();
        if ("NO_GUMGANLP".equalsIgnoreCase(basePackage)){
            log.warn("GumgaNLP ------ DISABLED");
            return;
        }
        try {
            ComponentFactory factory = ComponentFactory.create(new Locale("pt", "BR"));
            cogroo = factory.createPipe();
            reflections = new Reflections(basePackage);
            log.info("ReflectionsConfiguration------->" + reflections.getConfiguration().getUrls());
            classOfInterest = reflections.getTypesAnnotatedWith(GumgaNLPThing.class);
            log.warn("GumgaNLP ------" + basePackage + "----->" + classOfInterest);
        } catch (Exception ex) {
            log.error("GumgaNLP ------" + basePackage + "-----> NOT WORKING IN THIS SYSTEM " + ex);
        }
    }

    private enum Estados {
        VERBO, SUBSTANTIVO, FIM, ATRIBUTOS, VALOR_ATRIBUTO
    };

    public List<Object> createObjectsFromDocument(String text, String instanceVerbs) throws Exception {
        List<String> verbs = Arrays.asList(instanceVerbs.split(","));
        Document document = new DocumentImpl();
        document.setText(text);
        cogroo.analyze(document);
        print(document);
        ArrayList<Object> toReturn = new ArrayList<>();
        Object currentObject = null;
        Field currentField = null;
        Estados estado = Estados.VERBO;

        for (Sentence sentence : document.getSentences()) {
            int i = 0;
            List<Token> tokens = sentence.getTokens();
            while (i < tokens.size()) {
                Token token = sentence.getTokens().get(i);
                if (token.getPOSTag().startsWith("v-")) {
                    for (String v : verbs) {
                        if (Arrays.asList(token.getLemmas()).contains(v)) {
                            estado = Estados.SUBSTANTIVO;
                        }
                    }
                } else if (estado == Estados.SUBSTANTIVO && token.getPOSTag().startsWith("n")) {
                    Class classe = null;
                    for (Class c : classOfInterest) {
                        GumgaNLPThing gumgaNPLThing = (GumgaNLPThing) c.getAnnotation(GumgaNLPThing.class);
                        String nome = (NO_NAME.equals(gumgaNPLThing.value()) ? c.getSimpleName() : gumgaNPLThing.value()).toLowerCase();
                        if (nome.equals(token.getLexeme())) {
                            currentObject = c.newInstance();
                            toReturn.add(currentObject);
                            estado = Estados.ATRIBUTOS;
                        }
                    }
                } else if (estado == Estados.ATRIBUTOS && token.getPOSTag().startsWith("n")) {
                    try {
                        currentField = mapAllFields(currentObject.getClass()).get(token.getLexeme());  //currentObject.getClass().getDeclaredField(token.getLexeme());tod
                        estado = Estados.VALOR_ATRIBUTO;
                    } catch (NullPointerException nfe) {

                    }
                } else if (estado == Estados.VALOR_ATRIBUTO && (!token.getPOSTag().startsWith("prp"))) {
                    try {
                        currentField.setAccessible(true);
                        currentField.set(currentObject, currentField.getType().getConstructor(String.class).newInstance(token.getLexeme().toLowerCase()));
                        estado = Estados.ATRIBUTOS;
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
                i++;
            }

        }
        return toReturn;

    }

    private Map<String, Field> mapAllFields(Class clazz) {
        Map<String, Field> toReturn;
        if (clazz.getSuperclass().equals(Object.class)) {
            toReturn = new HashMap<>();
        } else {
            toReturn = mapAllFields(clazz.getSuperclass());
        }
        Field[] declaredFields = clazz.getDeclaredFields();
        for (Field f : declaredFields) {
            String label = f.getName();
            if (f.isAnnotationPresent(GumgaNLPThing.class)) {
                GumgaNLPThing annotation = f.getAnnotation(GumgaNLPThing.class);
                label = annotation.value();
            }
            toReturn.put(label, f);
        }
        return toReturn;

    }

    private void print(Document document) {
        StringBuilder output = new StringBuilder();
        for (Sentence sentence : document.getSentences()) {
            output.append("Sentence: ").append(sentence.getText()).append("\n");
            output.append("  Tokens: \n");
            for (Token token : sentence.getTokens()) {
                String lexeme = token.getLexeme();
                String lemmas = Arrays.toString(token.getLemmas());
                String pos = token.getPOSTag();
                String feat = token.getFeatures();
                output.append(String.format("    %-10s %-12s %-6s %-10s\n", lexeme, lemmas, pos, feat));
            }
            output.append("  Chunks: ");
            for (Chunk chunk : sentence.getChunks()) {
                output.append("[").append(chunk.getTag()).append(": ");
                for (Token innerToken : chunk.getTokens()) {
                    output.append(innerToken.getLexeme()).append(" ");
                }
                output.append("] ");
            }
            output.append("\n");
            output.append("  Shallow Structure: ");
            for (SyntacticChunk structure : sentence.getSyntacticChunks()) {
                output.append("[").append(structure.getTag()).append(": ");
                for (Token innerToken : structure.getTokens()) {
                    output.append(innerToken.getLexeme()).append(" ");
                }
                output.append("] ");
            }
            output.append("\n");
        }
        System.out.println(output.toString());
    }

}

==> gumga-application-3.1.0-sources.jar: io/gumga/application/nlp/GumgaNLP.java <==
package io.gumga.application.nlp;

import io.gumga.core.GumgaValues;
import io.gumga.domain.nlp.GumgaNLPThing;
import org.cogroo.analyzer.Analyzer;
import org.cogroo.analyzer.ComponentFactory;
import org.cogroo.text.*;
import org.cogroo.text.impl.DocumentImpl;
import org.reflections.Reflections;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Classe para processamento de linguagem natural
 */
@Component
public class GumgaNLP {
    
    public static final String NO_GUMGANLP = "NO_GUMGANLP";

    public static final String NO_NAME = "_NO_NAME";
    private Analyzer cogroo;
    private Reflections reflections;
    private Set<Class<?>> classOfInterest;
    private static final Logger log = LoggerFactory.getLogger(GumgaNLP.class);

    /**
     * Inicia o uso do processamento de linguagem natural
     * @param gumgaValues Objeto de configurações
     */
    @Autowired
    public GumgaNLP(GumgaValues gumgaValues) {
        String basePackage = gumgaValues.getGumgaNLPBasePackage();
        if ("NO_GUMGANLP".equalsIgnoreCase(basePackage)){
            log.warn("GumgaNLP ------ DISABLED");
            return;
        }
        try {
            ComponentFactory factory = ComponentFactory.create(new Locale("pt", "BR"));
            cogroo = factory.createPipe();
            reflections = new Reflections(basePackage);
            log.info("ReflectionsConfiguration------->" + reflections.getConfiguration().getUrls());
            classOfInterest = reflections.getTypesAnnotatedWith(GumgaNLPThing.class);
            log.warn("GumgaNLP ------" + basePackage + "----->" + classOfInterest);
        } catch (Exception ex) {
            log.error("GumgaNLP ------" + basePackage + "-----> NOT WORKING IN THIS SYSTEM " + ex);
        }
    }

    private enum Estados {
        VERBO, SUBSTANTIVO, FIM, ATRIBUTOS, VALOR_ATRIBUTO
    };

    /**
     * Cria Lista de ocorrências de objetos a serem instânciados
     * @param text Documento de linguagem natural, exemplo: crie uma casa de cor amarela...
     * @param instanceVerbs Verbos que serão utilizados para criação das instâncias, exemplo: Criar, Fazer...
     * @return Lista de objetos instanciados
     * @throws Exception
     */
    public List<Object> createObjectsFromDocument(String text, String instanceVerbs) throws Exception {
        List<String> verbs = Arrays.asList(instanceVerbs.split(","));
        Document document = new DocumentImpl();
        document.setText(text);
        cogroo.analyze(document);
        print(document);
        ArrayList<Object> toReturn = new ArrayList<>();
        Object currentObject = null;
        Field currentField = null;
        Estados estado = Estados.VERBO;

        for (Sentence sentence : document.getSentences()) {
            int i = 0;
            List<Token> tokens = sentence.getTokens();
            while (i < tokens.size()) {
                Token token = sentence.getTokens().get(i);
                if (token.getPOSTag().startsWith("v-")) {
                    for (String v : verbs) {
                        if (Arrays.asList(token.getLemmas()).contains(v)) {
                            estado = Estados.SUBSTANTIVO;
                        }
                    }
                } else if (estado == Estados.SUBSTANTIVO && token.getPOSTag().startsWith("n")) {
                    Class classe = null;
                    for (Class c : classOfInterest) {
                        GumgaNLPThing gumgaNPLThing = (GumgaNLPThing) c.getAnnotation(GumgaNLPThing.class);
                        String nome = (NO_NAME.equals(gumgaNPLThing.value()) ? c.getSimpleName() : gumgaNPLThing.value()).toLowerCase();
                        if (nome.equals(token.getLexeme())) {
                            currentObject = c.newInstance();
                            toReturn.add(currentObject);
                            estado = Estados.ATRIBUTOS;
                        }
                    }
                } else if (estado == Estados.ATRIBUTOS && token.getPOSTag().startsWith("n")) {
                    try {
                        currentField = mapAllFields(currentObject.getClass()).get(token.getLexeme());  //currentObject.getClass().getDeclaredField(token.getLexeme());tod
                        estado = Estados.VALOR_ATRIBUTO;
                    } catch (NullPointerException nfe) {

                    }
                } else if (estado == Estados.VALOR_ATRIBUTO && (!token.getPOSTag().startsWith("prp"))) {
                    try {
                        currentField.setAccessible(true);
                        currentField.set(currentObject, currentField.getType().getConstructor(String.class).newInstance(token.getLexeme().toLowerCase()));
                        estado = Estados.ATRIBUTOS;
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
                i++;
            }

        }
        return toReturn;

    }

    /**
     * Mapa de atributos da classe
     * @param clazz Classe
     * @return Mapa de atributos
     */
    private Map<String, Field> mapAllFields(Class clazz) {
        Map<String, Field> toReturn;
        if (clazz.getSuperclass().equals(Object.class)) {
            toReturn = new HashMap<>();
        } else {
            toReturn = mapAllFields(clazz.getSuperclass());
        }
        Field[] declaredFields = clazz.getDeclaredFields();
        for (Field f : declaredFields) {
            String label = f.getName();
            if (f.isAnnotationPresent(GumgaNLPThing.class)) {
                GumgaNLPThing annotation = f.getAnnotation(GumgaNLPThing.class);
                label = annotation.value();
            }
            toReturn.put(label, f);
        }
        return toReturn;

    }

    /**
     * Imprime log da criação de ocorrências NLP
     * @param document Documento NLP
     */
    private void print(Document document) {
        StringBuilder output = new StringBuilder();
        for (Sentence sentence : document.getSentences()) {
            output.append("Sentence: ").append(sentence.getText()).append("\n");
            output.append("  Tokens: \n");
            for (Token token : sentence.getTokens()) {
                String lexeme = token.getLexeme();
                String lemmas = Arrays.toString(token.getLemmas());
                String pos = token.getPOSTag();
                String feat = token.getFeatures();
                output.append(String.format("    %-10s %-12s %-6s %-10s\n", lexeme, lemmas, pos, feat));
            }
            output.append("  Chunks: ");
            for (Chunk chunk : sentence.getChunks()) {
                output.append("[").append(chunk.getTag()).append(": ");
                for (Token innerToken : chunk.getTokens()) {
                    output.append(innerToken.getLexeme()).append(" ");
                }
                output.append("] ");
            }
            output.append("\n");
            output.append("  Shallow Structure: ");
            for (SyntacticChunk structure : sentence.getSyntacticChunks()) {
                output.append("[").append(structure.getTag()).append(": ");
                for (Token innerToken : structure.getTokens()) {
                    output.append(innerToken.getLexeme()).append(" ");
                }
                output.append("] ");
            }
            output.append("\n");
        }
        System.out.println(output.toString());
    }

}
